package {
	import flash.display. * ;
	import flash.events. * ;
	import flash.geom. * ;
	import flash.filters. * ;

	public class ImageDetector {
		private var s: BitmapData;
		private var r: BitmapData;
		private var blocks: Array;
		
		public var minBlockSize:Point;
		public var maxBlockSize:Point;

		public function ImageDetector(minBlockW:uint, minBlockH:uint, maxBlockW:uint, maxBlockH:uint) {
			minBlockSize = new Point(minBlockW, minBlockH);
			maxBlockSize = new Point(maxBlockW, maxBlockH);
		}

		public function process(s: BitmapData):Array {
			var r1:BitmapData = getDiffImage(s);
			var r2:Array = getBlocks(r1); // [r:BitmapData, blocks:Array]
			return [r1, r2[0], r2[1]];
		}

		public function getDiffImage(s: BitmapData) : BitmapData {
			var r: BitmapData = new BitmapData(s.width, s.height);
			var r2: BitmapData = new BitmapData(s.width, s.height);
			var rect: Rectangle = new Rectangle(0, 0, s.width, s.height);
			var pt: Point = new Point(0, 0);
			r.draw(s);
			r2.draw(s);
			var mtx = new Matrix();
			mtx.translate(2, 0);
			r.draw(s, mtx, new ColorTransform(), BlendMode.DIFFERENCE);
			mtx.translate(-2, 2);
			r2.draw(s, mtx, new ColorTransform(), BlendMode.DIFFERENCE);

			r.copyPixels(r, new Rectangle(2, 0, 2, s.height), pt);
			r2.copyPixels(r2, new Rectangle(0, 2, s.width, 2), pt);

			r.draw(r2, new Matrix(), new ColorTransform(), BlendMode.ADD);

			var filter: ConvolutionFilter = new ConvolutionFilter(3, 3,
			[0.1, 0.1, 0.1,
			0.1, 0.1, 0.1,
			0.1, 0.1, 0.1]);
			r.applyFilter(r, rect, pt, filter);
			r.threshold(r, rect, pt, "<", 0xff111111, 0xff000000);
			r.threshold(r, rect, pt, "!=", 0xff000000, 0xffffffff);
			return r;
		}

		public function getBlocks(s: BitmapData):Array {
			blocks = new Array();
			var i: uint = 0;
			var o: Object;
			var size: uint;
			var r: BitmapData = new BitmapData(s.width, s.height);
			r.draw(s);
			var sp: Sprite = new Sprite();
			var g: Graphics = sp.graphics;
			var rect1: Rectangle,
			rect2: Rectangle;
			var MARK_COLOR:uint = 0xffff00ff, FILL_COLOR:uint = 0xff0099ff, RECT_COLOR:uint = 0x00FF00;
			
			while (true) {
				i++;
				if (i > 1000) {
					break;
				}
				rect1 = r.getColorBoundsRect(0xffffffff, 0xffffffff);
				if (rect1.isEmpty()) {
					break;
				}

				var x = rect1.x;
				for (var y: uint = rect1.y; y < rect1.y + rect1.height; y++) {
					if (r.getPixel32(x, y) == 0xffffffff) {
						r.floodFill(x, y, MARK_COLOR);
						rect2 = r.getColorBoundsRect(0xffffffff, MARK_COLOR);
						//size = rect2.width * rect2.height;
						if (rect2.width >= minBlockSize.x && rect2.height >= minBlockSize.y && 
							  rect2.width <= maxBlockSize.x && rect2.height <= maxBlockSize.y)
						{
							o = new Object();
							//o.size = size;
							o.rect = rect2;
							o.bmpd = new BitmapData(o.rect.width, o.rect.height);
							o.bmpd.copyPixels(r, o.rect, new Point(0, 0));
							blocks.push(o);
						}
						r.floodFill(x, y, FILL_COLOR);
					}
				}
			}

			for (i = 0; i < blocks.length; i++) {
				g.lineStyle(0, RECT_COLOR);
				g.drawRect(blocks[i].rect.x, blocks[i].rect.y, blocks[i].rect.width, blocks[i].rect.height);
			}

			r.draw(sp);
			return [r, blocks];
		}
	}
}
